package javax.swing.plaf.synth;

import java.beans.PropertyChangeEvent;
import java.awt.Graphics;
import java.awt.Dimension;
import javax.swing.Icon;
import java.awt.Insets;
import javax.swing.plaf.UIResource;
import javax.swing.LookAndFeel;
import sun.swing.MenuItemLayoutHelper;
import javax.swing.JMenuItem;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.basic.BasicMenuItemUI;

public class SynthMenuItemUI extends BasicMenuItemUI implements PropertyChangeListener, SynthUI
{
    private SynthStyle style;
    private SynthStyle accStyle;
    
    public SynthMenuItemUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new SynthMenuItemUI();
    }
    
    @Override
    public void uninstallUI(JComponent c) {
        super.uninstallUI(c);
        JComponent menuItemParent = MenuItemLayoutHelper.getMenuItemParent((JMenuItem)c);
        if (menuItemParent != null) {
            menuItemParent.putClientProperty(SynthMenuItemLayoutHelper.MAX_ACC_OR_ARROW_WIDTH, null);
        }
    }
    
    @Override
    protected void installDefaults() {
        this.updateStyle(this.menuItem);
    }
    
    @Override
    protected void installListeners() {
        super.installListeners();
        this.menuItem.addPropertyChangeListener(this);
    }
    
    private void updateStyle(JMenuItem c) {
        SynthContext context = this.getContext(c, 1);
        SynthStyle style = this.style;
        this.style = SynthLookAndFeel.updateStyle(context, this);
        if (style != this.style) {
            String propertyPrefix = this.getPropertyPrefix();
            Object value = this.style.get(context, propertyPrefix + ".textIconGap");
            if (value != null) {
                LookAndFeel.installProperty(c, "iconTextGap", value);
            }
            this.defaultTextIconGap = c.getIconTextGap();
            if (this.menuItem.getMargin() == null || this.menuItem.getMargin() instanceof UIResource) {
                Insets empty_UIRESOURCE_INSETS = (Insets)this.style.get(context, propertyPrefix + ".margin");
                if (empty_UIRESOURCE_INSETS == null) {
                    empty_UIRESOURCE_INSETS = SynthLookAndFeel.EMPTY_UIRESOURCE_INSETS;
                }
                this.menuItem.setMargin(empty_UIRESOURCE_INSETS);
            }
            this.acceleratorDelimiter = this.style.getString(context, propertyPrefix + ".acceleratorDelimiter", "+");
            this.arrowIcon = this.style.getIcon(context, propertyPrefix + ".arrowIcon");
            this.checkIcon = this.style.getIcon(context, propertyPrefix + ".checkIcon");
            if (style != null) {
                this.uninstallKeyboardActions();
                this.installKeyboardActions();
            }
        }
        context.dispose();
        SynthContext context2 = this.getContext(c, Region.MENU_ITEM_ACCELERATOR, 1);
        this.accStyle = SynthLookAndFeel.updateStyle(context2, this);
        context2.dispose();
    }
    
    @Override
    protected void uninstallDefaults() {
        SynthContext context = this.getContext(this.menuItem, 1);
        this.style.uninstallDefaults(context);
        context.dispose();
        this.style = null;
        SynthContext context2 = this.getContext(this.menuItem, Region.MENU_ITEM_ACCELERATOR, 1);
        this.accStyle.uninstallDefaults(context2);
        context2.dispose();
        this.accStyle = null;
        super.uninstallDefaults();
    }
    
    @Override
    protected void uninstallListeners() {
        super.uninstallListeners();
        this.menuItem.removePropertyChangeListener(this);
    }
    
    @Override
    public SynthContext getContext(JComponent component) {
        return this.getContext(component, this.getComponentState(component));
    }
    
    SynthContext getContext(JComponent component, int n) {
        return SynthContext.getContext(component, this.style, n);
    }
    
    SynthContext getContext(JComponent component, Region region) {
        return this.getContext(component, region, this.getComponentState(component, region));
    }
    
    private SynthContext getContext(JComponent component, Region region, int n) {
        return SynthContext.getContext(component, region, this.accStyle, n);
    }
    
    private int getComponentState(JComponent component) {
        int componentState;
        if (!component.isEnabled()) {
            componentState = 8;
        }
        else if (this.menuItem.isArmed()) {
            componentState = 2;
        }
        else {
            componentState = SynthLookAndFeel.getComponentState(component);
        }
        if (this.menuItem.isSelected()) {
            componentState |= 0x200;
        }
        return componentState;
    }
    
    private int getComponentState(JComponent component, Region region) {
        return this.getComponentState(component);
    }
    
    @Override
    protected Dimension getPreferredMenuItemSize(JComponent component, Icon icon, Icon icon2, int n) {
        SynthContext context = this.getContext(component);
        SynthContext context2 = this.getContext(component, Region.MENU_ITEM_ACCELERATOR);
        Dimension preferredMenuItemSize = SynthGraphicsUtils.getPreferredMenuItemSize(context, context2, component, icon, icon2, n, this.acceleratorDelimiter, MenuItemLayoutHelper.useCheckAndArrow(this.menuItem), this.getPropertyPrefix());
        context.dispose();
        context2.dispose();
        return preferredMenuItemSize;
    }
    
    @Override
    public void update(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        SynthLookAndFeel.update(context, graphics);
        this.paintBackground(context, graphics, component);
        this.paint(context, graphics);
        context.dispose();
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        this.paint(context, graphics);
        context.dispose();
    }
    
    protected void paint(SynthContext synthContext, Graphics graphics) {
        SynthContext context = this.getContext(this.menuItem, Region.MENU_ITEM_ACCELERATOR);
        String propertyPrefix = this.getPropertyPrefix();
        SynthGraphicsUtils.paint(synthContext, context, graphics, this.style.getIcon(synthContext, propertyPrefix + ".checkIcon"), this.style.getIcon(synthContext, propertyPrefix + ".arrowIcon"), this.acceleratorDelimiter, this.defaultTextIconGap, this.getPropertyPrefix());
        context.dispose();
    }
    
    void paintBackground(SynthContext synthContext, Graphics graphics, JComponent component) {
        SynthGraphicsUtils.paintBackground(synthContext, graphics, component);
    }
    
    @Override
    public void paintBorder(SynthContext synthContext, Graphics graphics, int n, int n2, int n3, int n4) {
        synthContext.getPainter().paintMenuItemBorder(synthContext, graphics, n, n2, n3, n4);
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if (SynthLookAndFeel.shouldUpdateStyle(propertyChangeEvent)) {
            this.updateStyle((JMenuItem)propertyChangeEvent.getSource());
        }
    }
}
